<!DOCTYPE html>
<html lang="en" xmlns="http://www.w3.org/1999/xhtml" xmlns:v="urn:schemas-microsoft-com:vml"
    xmlns:o="urn:schemas-microsoft-com:office:office">

<head>
    <meta charset="utf-8"> <!-- utf-8 works for most cases -->
    <meta name="viewport" content="width=device-width"> <!-- Forcing initial-scale shouldn't be necessary -->
    <meta http-equiv="X-UA-Compatible" content="IE=edge"> <!-- Use the latest (edge) version of IE rendering engine -->
    <meta name="x-apple-disable-message-reformatting"> <!-- Disable auto-scale in iOS 10 Mail entirely -->
    <meta http-equiv="Content-Type" content="text/html charset=UTF-8" />
    <title>{{ email_title }}</title> <!-- The title tag shows in email notifications, like Android 4.4. -->


    <style type="text/css">
        table {
            border-collapse: separate;
        }

        a,
        a:link,
        a:visited {
            text-decoration: none;
            color: #00788a;
        }

        a:hover {
            text-decoration: underline;
        }

        h2,
        h2 a,
        h2 a:visited,
        h3,
        h3 a,
        h3 a:visited,
        h4,
        h5,
        h6,
        .t_cht {
            color: #000 !important;
        }

        .ExternalClass p,
        .ExternalClass span,
        .ExternalClass font,
        .ExternalClass td {
            line-height: 100%;
        }

        .ExternalClass {
            width: 100%;
        }
    </style>
    </style>
    <!-- CSS Reset : BEGIN -->
    <style>
        /* What it does: Remove spaces around the email design added by some email clients. */
        /* Beware: It can remove the padding / margin and add a background color to the compose a reply window. */
        html,
        body {
            margin: 0 auto !important;
            padding: 0 !important;
            height: 100% !important;
            width: 100% !important;
        }

        /* What it does: Stops email clients resizing small text. */
        * {
            -ms-text-size-adjust: 100%;
            -webkit-text-size-adjust: 100%;
        }

        /* What it does: Centers email on Android 4.4 */
        div[style*="margin: 16px 0"] {
            margin: 0 !important;
        }

        /* What it does: Stops Outlook from adding extra spacing to tables. */
        table,
        td {
            mso-table-lspace: 0pt !important;
            mso-table-rspace: 0pt !important;
        }

        /* What it does: Fixes webkit padding issue. Fix for Yahoo mail table alignment bug. Applies table-layout to the first 2 tables then removes for anything nested deeper. */
        table {
            border-spacing: 0 !important;
            border-collapse: collapse !important;
            table-layout: fixed !important;
            margin: 0 auto !important;
        }

        table table table {
            table-layout: auto;
        }

        /* What it does: Uses a better rendering method when resizing images in IE. */
        img {
            -ms-interpolation-mode: bicubic;
        }

        /* What it does: A work-around for email clients meddling in triggered links. */
        *[x-apple-data-detectors],
        /* iOS */
        .x-gmail-data-detectors,
        /* Gmail */
        .x-gmail-data-detectors *,
        .aBn {
            border-bottom: 0 !important;
            cursor: default !important;
            color: inherit !important;
            text-decoration: none !important;
            font-size: inherit !important;
            font-family: inherit !important;
            font-weight: inherit !important;
            line-height: inherit !important;
        }

        /* What it does: Prevents Gmail from displaying an download button on large, non-linked images. */
        .a6S {
            display: none !important;
            opacity: 0.01 !important;
        }

        /* If the above doesn't work, add a .g-img class to any image in question. */
        img.g-img+div {
            display: none !important;
        }

        /* What it does: Prevents underlining the button text in Windows 10 */
        .button-link {
            text-decoration: none !important;
        }

        /* What it does: Removes right gutter in Gmail iOS app: https://github.com/TedGoas/Cerberus/issues/89  */
        /* Create one of these media queries for each additional viewport size you'd like to fix */

        /* iPhone 4, 4S, 5, 5S, 5C, and 5SE */
        @media only screen and (min-device-width: 320px) and (max-device-width: 374px) {
            .email-container {
                min-width: 320px !important;
            }
        }

        /* iPhone 6, 6S, 7, 8, and X */
        @media only screen and (min-device-width: 375px) and (max-device-width: 413px) {
            .email-container {
                min-width: 375px !important;
            }
        }

        /* iPhone 6+, 7+, and 8+ */
        @media only screen and (min-device-width: 414px) {
            .email-container {
                min-width: 414px !important;
            }
        }

        /**
   * Google webfonts. Recommended to include the .woff version for cross-client compatibility.
   */
        @media screen {
            @font-face {
                font-family: 'Source Sans Pro';
                font-style: normal;
                font-weight: 400;
                src: local('Source Sans Pro Regular'), local('SourceSansPro-Regular'), url(https://fonts.gstatic.com/s/sourcesanspro/v10/ODelI1aHBYDBqgeIAH2zlBM0YzuT7MdOe03otPbuUS0.woff) format('woff');
            }

            @font-face {
                font-family: 'Source Sans Pro';
                font-style: normal;
                font-weight: 700;
                src: local('Source Sans Pro Bold'), local('SourceSansPro-Bold'), url(https://fonts.gstatic.com/s/sourcesanspro/v10/toadOcfmlt9b38dHJxOBGFkQc6VGVFSmCnC_l7QZG60.woff) format('woff');
            }
        }

        /**
   * Avoid browser level font resizing.
   * 1. Windows Mobile
   * 2. iOS / OSX
   */
        body,
        table,
        td,
        a {
            -ms-text-size-adjust: 100%;
            /* 1 */
            -webkit-text-size-adjust: 100%;
            /* 2 */
        }

        /**
   * Remove extra space added to tables and cells in Outlook.
   */
        table,
        td {
            mso-table-rspace: 0pt;
            mso-table-lspace: 0pt;
        }

        /**
   * Better fluid images in Internet Explorer.
   */
        img {
            -ms-interpolation-mode: bicubic;
        }

        /**
   * Remove blue links for iOS devices.
   */
        a[x-apple-data-detectors] {
            font-family: inherit !important;
            font-size: inherit !important;
            font-weight: inherit !important;
            line-height: inherit !important;
            color: inherit !important;
            text-decoration: none !important;
        }

        /**
   * Fix centering issues in Android 4.4.
   */
        div[style*="margin: 16px 0;"] {
            margin: 0 !important;
        }

        body {
            width: 100% !important;
            height: 100% !important;
            padding: 0 !important;
            margin: 0 !important;
        }

        /**
   * Collapse table borders to avoid space between cells.
   */
        table {
            border-collapse: collapse !important;
        }

        a {
            color: #1a82e2;
        }

        img {
            height: auto;
            line-height: 100%;
            text-decoration: none;
            border: 0;
            outline: none;
        }
    </style>
    <!-- CSS Reset : END -->

    <style>
        .fluid {
            width: 100%;
        }

        .fixed {
            width: 800px;
        }

        #body-table {
            width: 100%;
            margin: auto;
            border-collapse: collapse;
            table-layout: fixed;
        }

        #body-table #white-row> .white-col{
            height: 25px;
            background-color: #eaf2ff;
        }

        #body-table #first-row>.spacer-left,
        #body-table #first-row>.spacer-right {
            height: 44px;
            background-color: white;
        }

        #body-table #title-row>.spacer-left,
        #body-table #title-row>.spacer-right {
            height: 50px;
            background-color: #eaf2ff;
        }

        #body-table #title-logo {
            position: relative;
            height: 90px;
            vertical-align: middle;
            text-align: center;
        }

        #body-table #content-row>.spacer-left,
        #body-table #content-row>.spacer-right {
            background-color: #eaf2ff;
        }

        #body-table #content-row>#body-content {
            position: relative;
            background-color: #fff;
            padding: 30px;
        }

        #body-table #spacer-row>.spacer-left,
        #body-table #spacer-row>.spacer-middle,
        #body-table #spacer-row>.spacer-right {
            height: 50px;
            background-color: #eaf2ff;
        }

        #body-table #footer-row {
            border-top: 1px solid #ccdff3;
        }

        #body-table #footer-row>.spacer-left,
        #body-table #footer-row>#footer-content,
        #body-table #footer-row>.spacer-right {
            background-color: #eaf2ff;
            padding-top: 15px;
        }

        #body-table #footer-row>#footer-content {
            padding-bottom: 40px !important;
        }

        #footer-content>p {
            font-size: 10px !important;
            color: #a4b3cb !important;
            margin-top: 0 !important;
        }
    </style>

</head>

<body width="100%" bgcolor="white" style="margin: 0; mso-line-height-rule: exactly;text-align: center;">


    <!-- Visually Hidden Preheader Text : BEGIN -->
    <div
        style="display: none; font-size: 1px; line-height: 1px; max-height: 0px; max-width: 0px; opacity: 0; overflow: hidden; mso-hide: all; font-family: sans-serif;">
        {% block email_hidden_header %}
        {% endblock %}
    </div>
    <!-- Visually Hidden Preheader Text : END -->

    <!-- Email Body : BEGIN -->
    <table id="body-table" role="presentation" cellspacing="0" cellpadding="0" border="0" align="center"
        class="email-container">
        <colgroup>
            <col class="fluid">
            <col class="fixed">
            <col class="fluid">
        </colgroup>
        <tr id="white-row">
            <td class="white-col"></td>
            <td class="white-col"></td>
            <td class="white-col"></td>
        </tr>
        <tr id="first-row">
            <td class="spacer-left"></td>
            <td rowspan="2" id="title-logo" style="background-color: #3c4859;">
                <img width="800px" alt="Atonra logo" border="0"
                    style="width: 800px; max-width: 800px; position: absolute; top: 0; left: 0;"
                    src="https://atonra.fra1.digitaloceanspaces.com/public/atonra_banderol.png" alt="Logo">
            </td>

            <td class="spacer-right"></td>
        </tr>
        <tr id="title-row">
            <td class="spacer-left"></td>
            <td class="spacer-right"></td>
        </tr>
        <tr id="content-row">
            <td class="spacer-left"></td>
            <td id="body-content">
                {% block body %}
                <p style="margin: 0;" style="font-size: 16px; color: #3c4859; text-align: center;"><b>{{salutation}}</b></p>
                {{content|safe}}
                {% endblock %}
            </td>
            <td class="spacer-right"></td>
        </tr>
        <tr id="spacer-row">
            <td class="spacer-left"></td>
            <td class="spacer-middle"></td>
            <td class="spacer-right"></td>
        </tr>
        <tr id="footer-row" style="background-color: #ccdff3; border-top: 1px solid #ccdff3;">
            <td class="spacer-left"></td>
            <td id="footer-content">
                <p style="margin: 0;" style="text-align: center">
                    <b>AtonRâ Partners SA</b><br />
                    7, rue de la Croix d’Or, 1204 Geneva, Switzerland<br />
                    T +41 22 906 16 16<br />
                    <a href="https://www.atonra.ch" target="">Website</a>
                </p>
                <p style="margin: 0;" style="text-align: center"><b>About AtonRâ Partners:</b> AtonRâ Partners is an asset management
                    company, founded in
                    2004 with head office in Geneva, incorporated under Swiss law, duly approved by the Swiss Financial
                    Market Supervisory Authority (FINMA) under the Swiss Collective Investment Schemes Act. AtonRâ
                    Partners is a conviction-driven asset manager combining industrial and scientific research with
                    financial analysis. AtonRâ Partners focuses on long-term trends powerful enough to be turned into
                    thematic equity portfolios.</p>
                {% if unsubscribe %}
                <p style="margin: 0;" style="text-align: center">If you receive this email because you are subscribed to our mailing list:
                    Please feel free to {{unsubscribe|safe}} </p>
                {% endif %}
            </td>
            <td class="spacer-right"></td>
        </tr>

    </table>
    <!-- Email Footer : END -->

</body>

</html>
